module NVD
  module Feed
    module CVE
      LOCATIONS = {} unless defined? CVE_LOCATIONS
      (["recent", "modified"] | (2002..Date.today.year).to_a).each do |key|
        LOCATIONS["cve_" + key.to_s] = "https://nvd.nist.gov/static/feeds/xml/cve/nvdcve-2.0-#{key.to_s}.xml"
      end

      def self.fetch(tag,etag=nil)
        retval = {}
        location = LOCATIONS[tag]
        return nil unless location
        etag = (NVD::Feed.etags[tag] || "" ) unless etag
        data = HTTParty.get(location, {:headers => {"If-None-Match" => etag}})
        NVD::Feed.etags[tag] = data.response.header["etag"]
        return NVD::Feed.etags[tag] if data.nil?
        if location.match(/gz$/)
          data = NVD::Feed.gunzip(data)
          if location.match(/\.xml/)
            xml = HTTParty::Parser.new(data, :xml).parse
            data = []
            binding.pry #unless xml["nvd"] && xml["nvd"]["entry"]
            xml["nvd"]["entry"].each do |item|
              # item = OpenStruct.new(item)
              item.each_pair do |k,v|
                v = v.detect {|i| i["lang"] == "en-US"} if v.is_a?(Array)
                item[k] = v["__content__"] if v["__content__"]
              end
              data << item
            end
            retval[tag] = data
          end
        end

        if data.class == HTTParty::Response
          response = []
          data["nvd"]["entry"].each do |item|
            item["cvss"].each do |k,v|
              v.each do |j,l|
                item["cvss"][k][j] = Time.parse(l) if j.to_s.include?("datetime")
                item["cvss"][k][j] = l.to_f if j.to_s.include?("score")
              end if v.is_a?(Hash)
            end if item["cvss"]
            refs = []
            [item["references"]].flatten.each do |r|
              r.each do |k,v|
                v = v.detect {|i| i["lang"] == "en-US"} if v.is_a?(Array)
                r[k] = v["href"] if v["__content__"] && v["href"]
              end if r
              refs << r
            end
            item["references"] = refs
            products = []
            products = [item["vulnerable_software_list"]["product"]].flatten if item["vulnerable_software_list"]
            item["vulnerable_software_list"] = products
            item.each do |k,v|
              item[k] = Time.parse(v) if k.to_s.include?("datetime")
            end
            response << item
          end
          retval = response
        end
        return retval
      end

      def self.fetch_all
        retval = {}
        LOCATIONS.each do |tag, location|
          puts tag
          retval[tag] = fetch(tag)
          return retval
        end
        retval
      end

      def self.save_off(name,data)

      end
    end

    @@feeds ||= []
    @@feeds << CVE

  end
end
